#!/usr/bin/python3

import datetime
import os
import subprocess

import gi

gi.require_version("Nautilus", "4.0")
from gi.repository import Nautilus, GObject


CFGPATH = os.path.expanduser("~/.plmgmt/playlistdir")
CUSTOMENV = None


def notify(title, text):
    subprocess.call(
        [
            "notify-send",
            "-a",
            "nautilus",
            title,
            text,
        ],
        env=CUSTOMENV,
    )


def run_add_song_to_month_playlist(files, playlist_filename):
    cmd = ["add-song-to-month-playlist"]
    cmd.append(playlist_filename)
    for file_obj in files:
        if file_obj.is_gone():
            continue
        cmd.append(
            os.path.abspath(
                file_obj.get_location().get_path(),
            )
        )
    ret = subprocess.call(cmd, env=CUSTOMENV)
    if ret != 0:
        notify(
            "Failed to add to playlist",
            "Program %s failed with error code %s" % (cmd[0], ret),
        )


class AddSongToMonthPlaylistItemExtension(GObject.GObject, Nautilus.MenuProvider):
    """Add the selected songs to the month's playlist."""

    def get_file_items(self, files):
        """Attaches context menu in Nautilus to the appropriate files."""
        if not files:
            return

        for file_obj in files:

            # Do not attach context menu to a directory
            if file_obj.is_directory():
                return

            # Do not attach context menu  to anything other that a file
            # local files only; not remote
            if file_obj.get_uri_scheme() != "file":
                return

            # Only attach context menu to audio / video files
            mime = file_obj.get_mime_type()
            if not mime.startswith("video") and not mime.startswith("audio"):
                return

        try:
            with open(CFGPATH) as f:
                pldir = f.read().strip()
        except Exception:
            # No configuration = no effect.
            return

        submenu = Nautilus.Menu()
        menu_item = Nautilus.MenuItem(
            name="AddSongToMonthPlaylistMenuProvider::AddSongToAPlaylist",
            label="Add to playlist...",
            tip="Adds the selected items to a specific playlist",
            icon="",
        )
        menu_item.set_submenu(submenu)
        latest = Nautilus.MenuItem(
            name="AddSongToMonthPlaylistMenuProvider::AddSongToMonthPlaylist",
            label="Add to month's playlist",
            tip="Adds the selected items to this month's playlist",
            icon="",
        )
        latest_pl = os.path.join(
            pldir, datetime.date.today().strftime("%Y-%m") + ".m3u"
        )
        latest.connect("activate", self.on_menuitem_clicked, files, latest_pl)
        submenu.append_item(latest)

        playlists = [
            z[1]
            for z in reversed(
                sorted(
                    [
                        (os.stat(x).st_mtime, x)
                        for x in [os.path.join(pldir, y) for y in os.listdir(pldir)]
                        if x.lower().endswith(".m3u")
                    ]
                )
            )
        ]
        for n in playlists:
            nfn = os.path.basename(n)
            entry = Nautilus.MenuItem(
                name=f"AddSongToMonthPlaylistMenuProvider::AddSongToPlaylist::{nfn}",
                label=f"Add to {nfn}",
                tip=f"Adds the selected items to the playlist {nfn}",
                icon="",
            )
            entry.connect("activate", self.on_menuitem_clicked, files, n)
            submenu.append_item(entry)

        return (menu_item,)

    def on_menuitem_clicked(self, menu, files, pl_path):
        """Called when user chooses files though Nautilus context menu."""
        return run_add_song_to_month_playlist(files, pl_path)
